/*This is a HPGL driver for HP650c plotter. The scale factors (FX FY) and
* the limits (lims[]) are setup for this device 
* This driver uses external setup files for user flexability.  The default
* files under $GRAPH/etc are colortable and width.650.  The user may define
* files under the current map $LOCATION/mapgen/$MNAME called .colortable
* .width650 and .offset.
*         Jill Schuler June 1994 
*/
#ifndef lint
static char *SCCSID = "@(#)Dhp650.c	OEMG v.3.2";
#endif
/* the following defines must be included for all drivers */
/* modified for multiple paper sizes */

/* if SUN 
#include <stdlib.h>
#include <stdio.h>
*/

#include <varargs.h>
#include <termio.h>
#include <stdio.h>
#include <math.h>
#include "plotter.h"
#define file stdout
#define ifile stdin

# define BAUD B9600
#define NUM_PEN 256
#define MAXNAMLEN 255
#define XPMAX lims[Dglobal.model_no].xlim
#define YPMAX lims[Dglobal.model_no].ylim


static char s[] = "\0";		/* required structure for return */
static XYS cursor = {
	0, 0, s};	/* data.  "s" may be larger, if needed */
/*	select appropriate baud rate for your system */
static int oldpen;
static char *spen;
static  float red[MAXPEN], green[MAXPEN], blue[MAXPEN], pens[MAXPEN];
FILE *pentmp;
static struct {
	int	xlim,  ylim;
} lims[7] = {
	9800, 6700,             /* A paper */
	15700, 10100,           /* B paper */
	20900, 15100,           /* C paper */
	33000, 20400,           /* D paper */
	43000, 33400,           /* E paper */
	43000, 33400,           /* direct connect */
        65024, 35560,           /* Ex Expanded x dimension 64in, y 35in*/
};

XYS *
Dhp650(va_alist) va_dcl {
	va_list vap;
	int cmd, i, pen=1, px1,py1,px2,py2;
	double Dx, Dy, Dt;
	long	x, y, t;
	XYS *ret = &cursor;
	static double FX = 1.0;
	static double FY = 1.0;
	static double XOFF = 200.0;
	static double YOFF = 200.0;

	FILE *facp, *faco;
	char *GRAPH, *LOCATION, *MNAME, facfile[100], offfile[100], buf[256];

	va_start(vap);
	cmd = va_arg(vap, int);
	switch(cmd) {
	case D_SCALE:
		if (Dglobal.scale <= 0.)
			Dglobal.scale = 1.;
		goto scaleit;
	case D_INIT:
		fputs("%-12345X@PJL",file);
		fputs("%-12345X@PJL ENTER LANGUAGE=HPGL2\n",file);
		fputs("^[%-1B\n",file);
		fputs("@PJL ENTER PALETTE = SOFTWARE\n",file);
		fputs("@PJL ENTER SIZE = SOFTWARE\n",file);
		fputs("@PJL ENTER ROTATE = 0\n",file);
		fputs("@PJL ENTER MARGINS = EXPAND\n",file);
		fputs("\033.I81;;17:\033.N;19:\n",file);
		fputs("SP1;PW0.35;PU;\n",file);
		fputs("BP;IN;IP;\n",file);
		fprintf(file,"PS%d,%d;\nNP255;\n",XPMAX,YPMAX);

		sethpgl_color();
		if ( Dglobal.model_no == 5 && !(isatty(fileno(file)) || isatty(fileno(file)))){
			fprintf(stderr,"Error: Paper size not specified\n");
			exit();
		}
		if (isatty(i = fileno(file))) { /* set baud rate */
			struct termio term;
			if (ioctl(i, TCGETA, &term) == -1)
				return NULL;
			term.c_iflag |= IXON+IXANY;
			term.c_cflag = (term.c_cflag & ~CBAUD) + BAUD;
			if (ioctl(i, TCSETA, &term)== -1)
				return NULL;
		}
		/* Get specific scale factors if available    */
		GRAPH = getenv("GRAPH");
		sprintf(facfile,"%s/etc/HP650",GRAPH);
		if ((facp = fopen (facfile,"r")) != NULL) {
			while ((fgets (buf,100,facp)) != NULL){
				if (buf[0] == '#' || buf[0] == ' ')
					continue;
				if ((sscanf(buf,"%lf %lf",&FX,&FY)) == 2)
					fprintf(stderr,"\nUsing Scale Factors:	X=%lf	Y=%lf\n",FX,FY);  
				else{
					fprintf(stderr,"\nReading HP650 scale file\n");
					fprintf(stderr,"Error: File Format Not Correct. Using Default\n");
					FX = 1.0;
					FY = 1.0;
				}
				break;
				fclose(facp);
			}
		}

		/* End of get factors   */
		/* Get specific offset factors if available    */
		LOCATION = getenv ("LOCATION");
		MNAME = getenv ("MNAME");
		sprintf (offfile,"%s/mapgen/%s/.offset",LOCATION,MNAME);
		if ((faco = fopen (offfile,"r")) != NULL){
                        while ((fgets (buf,100,faco)) != NULL){
                                if (buf[0] == '#' || buf[0] == ' ')
                                        continue;
				if ((sscanf(buf,"%lf %lf",&XOFF,&YOFF)) == 2)
					fprintf(stderr,"\nUsing Offset Factors:	X=%lf	Y=%lf\n",XOFF,YOFF);  
				else{
					fprintf(stderr,"\nReading OFFSET file\n");
                                        fprintf(stderr,"Error: File Format Not Correct. Using Default\n");
                                        XOFF = 0.0;
                                        YOFF = 0.0;
                        	}
                        	break;
				fclose(faco);
			}
		}

		/* End of get factors   */
/*		fputs("\033.I81;;17:\033.N;19:\n",file);*/


	oldpen = 1;
	if (Dglobal.model_no == 5) {
		struct termio term;
		if (ioctl(i, TCGETA, &term) == -1)
			return NULL;
		term.c_iflag |= IXON+IXANY;
		term.c_cflag = (term.c_cflag & ~CBAUD) + BAUD;
		if (ioctl(i, TCSETA, &term)== -1)
			return NULL;
		fputs("OP;\n",file);
		scanf("%d,%d,%d,%d",&px1,&py1,&px2,&py2);
		lims[Dglobal.model_no].xlim = px2 - px1 +300;
		lims[Dglobal.model_no].ylim = py2 - py1 +300;
	}
scaleit:
	if (Dglobal.reverse) {
		cursor.x = lims[Dglobal.model_no].ylim / Dglobal.scale * 2;
		cursor.y = lims[Dglobal.model_no].xlim / Dglobal.scale * 2;
	} else {
		cursor.x = lims[Dglobal.model_no].xlim / Dglobal.scale * 2;
		cursor.y = lims[Dglobal.model_no].ylim / Dglobal.scale * 2;
	}
	break;
case D_DONE:
	fputs("SP;\nPG;\n",file);
	fputs("%-12345X\n",file);
	fputs("%-12345X@PJL RESET\n",file);
	fputs("@PJL EOJ NAME = \"JOB:; User: MAPGEN\"\n",file);
	fputs("%-12345X\n",file);
	fclose(file);
	break;
case D_MOVE:
	spen = "PU";
	goto moveit;
case D_LINE:
	spen = "PD";
moveit:
	Dx = (va_arg(vap, long)) * Dglobal.scale * 2.0;
	Dy = (va_arg(vap, long)) * Dglobal.scale * 2.0;

	if (Dglobal.reverse) {
		Dt = Dx;
		Dx = -(Dy - (lims[Dglobal.model_no].xlim / 2.0) + YOFF);
		Dy = Dt - (lims[Dglobal.model_no].ylim / 2.0) + XOFF;
	} else {
		Dx = Dx - (lims[Dglobal.model_no].xlim ) + XOFF;
		Dy = Dy - (lims[Dglobal.model_no].ylim ) + YOFF;
	}
	x = floor(Dx * FX);
	y = floor(Dy * FY);
	fprintf(file,"%s%d,%d;\n", spen, -x, -y);
	break;
case D_PEN:
	pen = (va_arg(vap, long));
	while (pen > NUM_PEN) pen = pen - NUM_PEN;
	if (pen == 0) pen = 1;
	if (pen != oldpen) {
		oldpen = pen;
		fprintf(file,"SP%d;PW%.3f;PU;\n",pen,pens[pen-1]);
	}
}

va_end(vap);
return ret;
}

/*************** START color handler *****************/
void sethpgl_color()
{
FILE *colortab, *widthfp;
int i,cg,cb,cr;
float x, y, ox, cw, oy;
char cname[50], name[MAXNAMLEN], *base, buff[MAXNAMLEN];

for (i=0;i<256;i++)
	{
	pens[i] = 0.35;
	}

/********** GET THE default and user specified PEN WIDTHS **********/
if ((base = getenv("GISBASE")) != NULL) 
{
        sprintf(buff,"%s/mapgen/etc/width.650",base);
        if ((widthfp = fopen(buff,"r")) != NULL) 
	{
		while (fscanf(widthfp,"%f %d",&cw,&i) != EOF)
		{
                        if (i < 256 && i > 0) 
			{ 
			pens[i-1] = cw;
                        }
                }
                fclose(widthfp);
        }
}
if ((widthfp = fopen(".width650","r")) != NULL) 
{
	while (fscanf(widthfp,"%f %d",&cw,&i) != EOF)
	{
		if (i < 256 && i > 0) 
		{
		pens[i-1] = cw;
		}
	}
	fclose(widthfp);
}

/************* GET the default then user specified PEN/color mapping **/
/* Note: this replaces color setup in driver and gives greater flexibility */
if ((colortab = fopen(".colortable","r")) != NULL) 
{
                while (fgets(name,MAXNAMLEN,colortab) != NULL) 
		{
                        sscanf(name,"%d %d %d %d %[^\n]",&cr,&cg,&cb,&i,cname);
                        if (i < 256 && i > 0)
			{
			fprintf(file,"PC%d,%d,%d,%d;\n",i,cr,cg,cb);
                        }
        	}
        	fclose(colortab);
}
else 
{
	if((base = getenv("GISBASE")) != NULL) 
	{
        sprintf(buff,"%s/mapgen/etc/colortable",base);
        	if ((colortab = fopen(buff,"r")) != NULL) 
		{
                while (fgets(name,MAXNAMLEN,colortab) != NULL) 
		{
                        sscanf(name,"%d %d %d %d",&cr,&cg,&cb,&i);
                        if (i < 256 && i > 0)
			{
			fprintf(file,"PC%d,%d,%d,%d;\n",i,cr,cg,cb);
                        }
                }
                fclose(colortab);
        	}
	}
}

} /* End of sethpgl_color function*/

/************* END color handler ***********/
