# ifndef lint
static char *SCCSID = "@(#)Dhp650cps.c	USGS ";
# endif
/*This is a cps driver for HP650c plotter. 
* the limits (lims[]) are setup for this device
* This driver uses external setup files for user flexability.  The default
* files under $GRAPH/etc are colortable and width.  The user may define
* files under the current map $LOCATION/mapgen/$MNAME called .colortable
* .width and .margin.
*/
/* psqms - driver for HP650c PostScript output */
# define PLOTTER

# include <varargs.h>
# include "plotter.h"

/*Added to set up more paper sizes*/
#define XPMAX sheet[Dglobal.model_no].xpmax
#define XPIN sheet[Dglobal.model_no].xpin
#define YPMAX sheet[Dglobal.model_no].ypmax
#define YPIN sheet[Dglobal.model_no].ypin

/* establish number of typesetter points/inch.
** The following is the legal definition is 72.27 pts/in but some machines
** appear to use 72 (even) */
#define PTS_PER_IN 72.0
/* Convert 200 counts/cm to points: 2.54 * 200 */
#define IN_TO_CNTS      508.0
#define MAXNAMLEN 255


static struct SHEET {
        float   ypmax, xpmax, ypin, xpin;   /* axis size of paper in inches: 508 x in */
} sheet[] = {
	4319.,	5588.,	792.,	612.,	/* 11x8.5" standard or A size */
	5588.,	8636.,	1224.,	792.,	/* 17x11" B size */
	8636.,	11176.,	1584.,	1224.,	/* 22x17" C size */
	11176.,	17272.,	2448.,	1584.,	/* 34x22" D size */
	17272.,	22352.,	3168.,	2448.,	/* 44x34" E size */
	18288.,	25908.,	3672.,	2592.,	/* 51x36" Ex size */
};


static char s[] = "\0";		/* required structure for return */
static XYS cursor = {0, 0, s};	/* data.  "s" may be larger, if needed */
#define MAX_DXY 50
	static XY
dxy[MAX_DXY];
	static int
nline = 0,
ndxy = 0,
troff = 0,
noshow = 0,
oldpen = 0,
pen,
pendown;
static	float red[MAXPEN], green[MAXPEN], blue[MAXPEN], pens[MAXPEN];

/* initialization strings and PostScript abbreviations */
/* \004 tag set to indicate beginning of PostScript data */
/* Changed scale value to new plotter count value 72/508 */

 static char
*initstr = "%%-12345X@PJL
%%-12345X@PJL ENTER LANGUAGE=postscript
%%!PS-Adobe-2.0 
%%%%Title: Untitled
%%%%Creator: GRASS*MAPGEN HP650c PostScript Driver
%%%%CreationDate:
%%%%Pages: (atend)
%%%%DocumentFonts: (atend)
%%%%BoundingBox: 0 0 %d %d
%%%%ProofMode: Substitute
%%%%EndComments
%%%%BeginProlog
%% ############ GRASS*MAPGEN PostScript Header  #################
%%
%%%%EndProlog
%%%%BeginSetup
%%%%Page: 1 1
/SC { 0.1417322 } def
/L { {rlineto} repeat currentpoint stroke moveto} bind def
/U { moveto } bind def
1 setlinecap
1 setlinejoin
%%%%EndSetup
gsave\n",
 
*init2 = " SC SC scale",

*end = "%%Trailer
%%DocumentFonts:
%%Pages: 1
%%EOF
%%-12345X
%%-12345X@PJL RESET
%%-12345X",
 
*landstr = "
0 %.1f translate
-90 rotate
",

*portstr = "";

	static void
dump() {
	int n;

	if (!ndxy) {
		if (pendown > 1)	return;
		dxy[0].x = dxy[0].y = 0;
		++ndxy;
	}
	n = ndxy;
	while (n--) /* print out in reverse order */
		if ((nline += printf("%d %d ",dxy[n].x,dxy[n].y)) > 70 && n) {
			(void)putchar('\n');
			nline = 0;
		}
	(void)printf("%d L\n",ndxy);
	nline = ndxy = 0;
	++pendown;
}
	static long
x, y, xl, yl, dx, dy;
	extern long
strtol();
long mX=60,mY=60;

/******** MAIN plot driver routine ***********/
	XYS *
Dhp650cps(va_alist) va_dcl {
	va_list vap;
	int cmd, i;
	long lpen;
	XYS *ret = &cursor;
	char *ss;

	va_start(vap); cmd = va_arg(vap, int);
	switch(cmd) {
	case D_SCALE:
		if (Dglobal.scale <= 0.)	Dglobal.scale = 1.;
		goto scaleit;
	case D_INIT:
		Set_color();
		/*Dglobal.scale = Dglobal.scale / 2.0; commented out for plotter = 400 */
		for (x = xl = mX, y = yl = mY, i = 1; i < Dglobal.dargc ; ++i)
			switch (*Dglobal.dargv[i]) {
			case 't':	/* troff mode */
				troff = 1;
				Dglobal.reverse = 1;
			case 'x':	/* suppress showpage */
				noshow = 1;
				break;
			case 'o':	/* offset -o<x>,<y> */
				ss = Dglobal.dargv[i];
				x = strtol(++ss, &ss, 0);
				y = strtol(++ss, &ss, 0);
				break;
			default:
				(void)fprintf(stderr,"unknown arg:%s\n",
					Dglobal.dargv[i]);
				ret = (XYS *)0;
			}
		if (!ret) break;
		pendown = 0;
		if (!troff)  printf(initstr,(int)XPIN,(int)YPIN,(int)YPIN,(int)XPIN);
		puts(init2);


                if (Dglobal.reverse)
                        puts(portstr);
                else
                        printf(landstr, XPMAX);
                if (x || y)     (void)printf("%d %d translate\n",x,y);


scaleit:
		if (Dglobal.reverse) {
			cursor.y = (troff ? 10000 : XPMAX) / Dglobal.scale;
			cursor.x = (troff ? 10000 : YPMAX) / Dglobal.scale;
		} else {
			cursor.x = XPMAX / Dglobal.scale;
			cursor.y = YPMAX / Dglobal.scale;
		}
		break;
	case D_DONE:
	case D_PANIC:
		if (pendown)	dump();
		if (!troff)	puts("grestore\n");
		if (!noshow)	puts("showpage\n\004\n");
		if (!troff)	puts(end);
		break;
	case D_MOVE:
		if (pendown) {
			dump();
			pendown = 0;
		}
	case D_LINE:
		x = (va_arg(vap, long)) * Dglobal.scale + .5;
		y = (va_arg(vap, long)) * Dglobal.scale + .5;
		if (pendown) {
			if ((dx = x - xl) | (dy = y - yl)) {
				dxy[ndxy].x = dx;
				dxy[ndxy].y = dy;
				if (++ndxy >= MAX_DXY) dump();
			}
		} else {
			nline = printf("%d %d U ", x, y);
			pendown = 1;
		}
		xl = x;
		yl = y;
		break;
	case D_PEN:
		lpen = va_arg(vap, long);
		pen = lpen;
		while (pen > MAXPEN) pen -= MAXPEN;
		if (pen < 1) pen = 1;
		if (pen != oldpen) {
			if (pendown)	dump();
			if (red[pen-1] == 0.0 && green[pen-1] == 0.0 && blue[pen-1] == 0.0)
				(void)printf("%.3f setlinewidth 0 0 0 1 setcmykcolor\n",
				pens[pen-1]);
			else (void)printf("%.3f setlinewidth %.3f %.3f %.3f setrgbcolor\n",
					pens[pen-1],
					red[pen-1],
					green[pen-1],
					blue[pen-1]);
			pendown = 0;
			oldpen = pen;
		}
		break;
	}
	va_end(vap);
	return ret;
}

/*************** START color handler *****************/
void Set_color()
{
FILE *colortab, *widthfp, *marfp;
int i,cg,cb,cr;
float x, y, ox, cw, oy;
/* char name[50]; */
char cname[50], name[MAXNAMLEN], *base, buff[MAXNAMLEN];

for (i=0;i<256;i++){
	pens[i] = 2.0;

	RESET_COLOR(0,0,0,i);
	}
/****** GET default then user specified margins for map *********/
if ((base = getenv("GISBASE")) != NULL) {
        sprintf(buff,"%s/mapgen/etc/margin.cps",base);
        if ((marfp = fopen(buff,"r")) != NULL) {
                while (fgets(name,MAXNAMLEN,marfp) != NULL){
                        sscanf(name,"%f %f",&x,&y);
                        }
		ox = mX; oy = mY;
		mX = (long)(ox + (200.0 * x) * Dglobal.scale);
		mY = (long)(oy + (200.0 * y) * Dglobal.scale);
                fclose(marfp);
                }
        }
if ((marfp = fopen(".margin","r")) != NULL) {
	fscanf(marfp,"%f %f",&x,&y);
	ox = mX; oy = mY;
	mX = (long)(ox + (200.0 * x) * Dglobal.scale);
	mY = (long)(oy + (200.0 * y) * Dglobal.scale);
	fclose(marfp);
	}

/********** GET THE default and user specified PEN WIDTHS **********/
if ((base = getenv("GISBASE")) != NULL) {
        sprintf(buff,"%s/mapgen/etc/width",base);
        if ((widthfp = fopen(buff,"r")) != NULL) {
                while (fgets(name,MAXNAMLEN,widthfp) != NULL){
                        sscanf(name,"%f %d",&cw,&i);
                        if (i < 256 && i > 0) pens[i-1] = cw;
                        }
                fclose(widthfp);
                }
        }
if ((widthfp = fopen(".width","r")) != NULL) {
	while (fscanf(widthfp,"%f %d",&cw,&i) != EOF)
		if (i < 256 && i > 0) pens[i-1] = cw;
	fclose(widthfp);
	}

/************* GET the default then user specified PEN/color mapping **/
/* Note: this replaces color setup in driver and gives greater flexibility */
if ((base = getenv("GISBASE")) != NULL) {
        sprintf(buff,"%s/mapgen/etc/colortable",base);
        if ((colortab = fopen(buff,"r")) != NULL) {
                while (fgets(name,MAXNAMLEN,colortab) != NULL){
                        sscanf(name,"%d %d %d %d",&cr,&cg,&cb,&i);
                        if (i < 256 && i > 0)
				RESET_COLOR(cr,cg,cb,i);
                        }
                fclose(colortab);
                }
        }
if ((colortab = fopen(".colortable","r")) != NULL) {
                while (fgets(name,MAXNAMLEN,colortab) != NULL){
                        sscanf(name,"%d %d %d %d %[^\n]",&cr,&cg,&cb,&i,cname);
                        if (i < 256 && i > 0)
				RESET_COLOR(cr,cg,cb,i);
                        }
        fclose(colortab);
        }
}



RESET_COLOR(r,g,b,i)
int r,g,b,i;
{
float a;
int j;

j = i -1;
a = (float)r/256.0;
red[j] = a;
a = (float)g/256.0;
green[j] = a;
a = (float)b/256.0;
blue[j] = a;
}
/* End color Handler*/
